/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

void SurfaceAccessorHoudini::addString(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addString"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}
void SurfaceAccessorHoudini::addInteger(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addInteger"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addReal(void)
{
//	feLog("SurfaceAccessorHoudini::addReal \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		const Real zero=0.0;
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addReal"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addSpatialVector(void)
{
//	feLog("SurfaceAccessorHoudini::addSpatialVector \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addSpatialVector"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

} // namespace
