/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "maya/maya.pmh"

namespace fe
{

I32 SurfaceAccessorMaya::createBlindDataID(String a_attrName,String a_attrType)
{
	const String attrKey=a_attrName+":"+a_attrType;
	I32& rBlindDataID=m_spMaster->catalog()->catalog<I32>(
			"MayaBlindID",attrKey,-1);

	if(rBlindDataID>=0)
	{
		//* created already
		return rBlindDataID;
	}

	feLog("SurfaceAccessorMaya::assureBlindDataID"
			" creating blind data for \"%s\"\n",attrKey.c_str());

	rBlindDataID=0x0;
	while(m_pFnMesh->isBlindDataTypeUsed(rBlindDataID))
	{
		rBlindDataID++;
	}
	feLog("SurfaceAccessorMaya::assureBlindDataID"
			" blindDataID %d\n",rBlindDataID);

	MStringArray longNames;
	MStringArray shortNames;
	MStringArray formatNames;

	// format names: int float double boolean string binary

	longNames.append(a_attrName.c_str());
	shortNames.append(a_attrName.c_str());
	if(a_attrType=="Real")
	{
		formatNames.append("double");
	}
	else if(a_attrType=="I32")
	{
		formatNames.append("int");
	}

	MStatus status=m_pFnMesh->createBlindDataType(
			rBlindDataID,
			longNames,shortNames,formatNames);
	if(!status)
	{
		feLog("SurfaceAccessorMaya::assureBlindDataID"
				" createBlindDataType %p failed\n",rBlindDataID);
	}

	return rBlindDataID;
}

} // namespace
