/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

using namespace fe;

void BloatOp::initialize(void)
{
	catalog<Real>("Scaling")=1.0;
	catalog<Real>("Scaling","min")= -100.0;
	catalog<Real>("Scaling","low")= 0.0;
	catalog<Real>("Scaling","high")=2.0;
	catalog<Real>("Scaling","max")=100.0;
	catalog<String>("Scaling","hint")=
			"Scale evenly around center, like a common transform.";

	catalog<Real>("Expansion")=0.0;
	catalog<Real>("Expansion","min")= -100.0;
	catalog<Real>("Expansion","low")= -1.0;
	catalog<Real>("Expansion","high")=1.0;
	catalog<Real>("Expansion","max")=100.0;
	catalog<String>("Expansion","hint")=
			"Scale away from center, with a uniform distance.";

	catalog<Real>("Bloat")=0.0;
	catalog<Real>("Bloat","min")= -100.0;
	catalog<Real>("Bloat","low")= -1.0;
	catalog<Real>("Bloat","high")=1.0;
	catalog<Real>("Bloat","max")=100.0;
	catalog<String>("Bloat","hint")=
			"Scale in direction of normals, with a uniform distance.";

	catalog< sp<Component> >("Input Surface");
}

void BloatOp::handle(Record& a_rSignal)
{
	const Real bloat=catalog<Real>("Bloat");
	const Real expansion=catalog<Real>("Expansion");
	const Real scaling=catalog<Real>("Scaling");

	//* access output surface (copy of input)
	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	if(spSurfaceAccessibleI.isNull())
	{
		feLog("BloatOp::handle output surface not spSurfaceAccessibleI\n");
		catalog<String>("error")="inaccessible output";
		return;
	}

	sp<SurfaceAccessorI> spAccessorPoint=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	FEASSERT(spAccessorPoint.isValid());

	const U32 count=spAccessorPoint->count();
	if(!count)
	{
		feLog("BloatOp::handle point count is zero\n");
		catalog<String>("error")="points required";
		return;
	}

	sp<SurfaceAccessorI> spAccessorNormal;
	if(bloat!=0.0)
	{
		spAccessorNormal=spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_normal);
		if(spAccessorNormal.isNull() || spAccessorNormal->count()!=count)
		{
//			feLog("BloatOp::handle access to normals failed\n");
			catalog<String>("warning")="normals required for non-zero bloat";
		}
	}

	if(expansion!=0.0 || scaling!=1.0)
	{
		SpatialVector min=spAccessorPoint->spatialVector(0);
		SpatialVector max=min;
		for(U32 index=1;index<count;index++)
		{
			const SpatialVector point=spAccessorPoint->spatialVector(index);

			for(U32 m=0;m<3;m++)
			{
				const Real value=point[m];
				if(min[m]>value)
				{
					min[m]=value;
				}
				if(max[m]<value)
				{
					max[m]=value;
				}
			}
		}
		const SpatialVector center=0.5*(min+max);

		for(U32 index=0;index<count;index++)
		{
			const SpatialVector point=spAccessorPoint->spatialVector(index);
			const SpatialVector normal=spAccessorNormal.isValid()?
					spAccessorNormal->spatialVector(index):
					SpatialVector(0.0,0.0,0.0);

			spAccessorPoint->set(index,scaling*(point-center)+center+
					unitSafe(point-center)*expansion+normal*bloat);
		}
	}
	else
	{
		for(U32 index=0;index<count;index++)
		{
			const SpatialVector point=spAccessorPoint->spatialVector(index);
			const SpatialVector normal=spAccessorNormal.isValid()?
					spAccessorNormal->spatialVector(index):
					SpatialVector(0.0,0.0,0.0);

			spAccessorPoint->set(index,point+normal*bloat);
		}
	}
}
